@@ -1,7 +1,136 @@
-use Modern::Perl;
-
 package DBIx::Squirrel;
-$DBIx::Squirrel::VERSION = '1.3.0'; # TRIAL
+
+use 5.010_001;
+use strict;
+use warnings;
+use DBI;
+use Exporter;
+use Scalar::Util qw/reftype/;
+use Sub::Name;
+use DBIx::Squirrel::dr          ();
+use DBIx::Squirrel::db          ();
+use DBIx::Squirrel::st          ();
+use DBIx::Squirrel::Iterator    ();
+use DBIx::Squirrel::ResultSet   ();
+use DBIx::Squirrel::ResultClass ();
+use DBIx::Squirrel::Utils       qw/throw/;
+use namespace::clean;
+
+BEGIN {
+    @DBIx::Squirrel::ISA                          = qw/DBI/;
+    $DBIx::Squirrel::VERSION                      = '1.4.1';
+    @DBIx::Squirrel::EXPORT_OK                    = @DBI::EXPORT_OK;
+    %DBIx::Squirrel::EXPORT_TAGS                  = %DBI::EXPORT_TAGS;
+    *DBIx::Squirrel::err                          = *DBI::err;
+    *DBIx::Squirrel::errstr                       = *DBI::errstr;
+    *DBIx::Squirrel::rows                         = *DBI::rows;
+    *DBIx::Squirrel::lasth                        = *DBI::lasth;
+    *DBIx::Squirrel::state                        = *DBI::state;
+    *DBIx::Squirrel::connect                      = *DBIx::Squirrel::dr::connect;
+    *DBIx::Squirrel::connect_cached               = *DBIx::Squirrel::dr::connect_cached;
+    *DBIx::Squirrel::FINISH_ACTIVE_BEFORE_EXECUTE = *DBIx::Squirrel::st::FINISH_ACTIVE_BEFORE_EXECUTE;
+    *DBIx::Squirrel::DEFAULT_SLICE                = *DBIx::Squirrel::Iterator::DEFAULT_SLICE;
+    *DBIx::Squirrel::DEFAULT_CACHE_SIZE           = *DBIx::Squirrel::Iterator::DEFAULT_CACHE_SIZE;
+    *DBIx::Squirrel::CACHE_SIZE_LIMIT             = *DBIx::Squirrel::Iterator::CACHE_SIZE_LIMIT;
+}
+
+use constant E_BAD_ENT_BIND     => 'Cannot associate with an invalid object';
+use constant E_EXP_HASH_ARR_REF => 'Expected a reference to a HASH or ARRAY';
+
+# Divide the argumments into two lists:
+# 1. a list of helper function names;
+# 2. a list of names to be imported from the DBI.
+sub _partition_imports_into_helpers_and_dbi_imports {
+    my(@helpers, @dbi);
+    while (@_) {
+        next unless defined($_[0]);
+        if ($_[0] =~ m/^database_entit(?:y|ies)$/i) {
+            shift;
+            if (ref($_[0])) {
+                if (UNIVERSAL::isa($_[0], 'ARRAY')) {
+                    push @helpers, @{+shift};
+                }
+                else {
+                    shift;
+                }
+            }
+            else {
+                push @helpers, shift();
+            }
+        }
+        else {
+            push @dbi, shift();
+        }
+    }
+    return (\@helpers, \@dbi);
+}
+
+sub import {
+    no strict 'refs';    ## no critic
+    my $class  = shift;
+    my $caller = caller;
+    my($helpers, $dbi) = _partition_imports_into_helpers_and_dbi_imports(@_);
+    for my $name (@{$helpers}) {
+        my $symbol = $class . '::' . $name;
+        my $helper = sub {
+            unless (defined(${$symbol})) {
+                if (@_) {
+                    throw E_BAD_ENT_BIND
+                      unless UNIVERSAL::isa($_[0], 'DBI::db')
+                      or UNIVERSAL::isa($_[0], 'DBI::st')
+                      or UNIVERSAL::isa($_[0], 'DBIx::Squirrel::Iterator');
+                    ${$symbol} = shift;
+                    return ${$symbol};
+                }
+            }
+            return unless defined(${$symbol});
+            if (@_) {
+                my @params = do {
+                    if (@_ == 1 && ref $_[0]) {
+                        if (reftype($_[0]) eq 'ARRAY') {
+                            @{+shift};
+                        }
+                        elsif (reftype($_[0]) eq 'HASH') {
+                            %{+shift};
+                        }
+                        else {
+                            throw E_EXP_HASH_ARR_REF;
+                        }
+                    }
+                    else {
+                        @_;
+                    }
+                };
+                if (UNIVERSAL::isa(${$symbol}, 'DBI::db')) {
+                    return ${$symbol}->prepare(@params);
+                }
+                elsif (UNIVERSAL::isa(${$symbol}, 'DBI::st')) {
+                    return ${$symbol}->execute(@params);
+                }
+                elsif (UNIVERSAL::isa(${$symbol}, 'DBIx::Squirrel::Iterator')) {
+                    return ${$symbol}->iterate(@params);
+                }
+                else {
+                    # ok - no worries
+                }
+            }
+            return ${$symbol};
+        };
+        *{$symbol} = subname($name => $helper);
+        *{$caller . '::' . $name} = subname($caller . '::' . $name => \&{$symbol})
+          unless defined(&{$caller . '::' . $name});
+    }
+    if (@{$dbi}) {
+        DBI->import(@{$dbi});
+        @_ = ('DBIx::Squirrel', @{$dbi});
+        goto &Exporter::import;
+    }
+    return $class;
+}
+
+1;
+__END__
+
 =pod
 
 =encoding UTF-8
@@ -12,7 +141,7 @@ DBIx::Squirrel - A C<DBI> extension
 
 =head1 VERSION
 
-version 1.3.0
+version 1.4.1
 
 =head1 SYNOPSIS
 
@@ -327,142 +456,6 @@ version 1.3.0
         },
     );
     $id = $itr->iterate('Acme Rocket')->single;
-=cut
-
-use DBI;
-use Exporter;
-use Scalar::Util qw/reftype/;
-use Sub::Name;
-use DBIx::Squirrel::dr   ();
-use DBIx::Squirrel::db   ();
-use DBIx::Squirrel::st   ();
-use DBIx::Squirrel::it   ();
-use DBIx::Squirrel::rs   ();
-use DBIx::Squirrel::rc   ();
-use DBIx::Squirrel::util qw/throw uniq/;
-
-BEGIN {
-    @DBIx::Squirrel::ISA                          = 'DBI';
-    *DBIx::Squirrel::EXPORT_OK                    = *DBI::EXPORT_OK;
-    *DBIx::Squirrel::EXPORT_TAGS                  = *DBI::EXPORT_TAGS;
-    *DBIx::Squirrel::err                          = *DBI::err;
-    *DBIx::Squirrel::errstr                       = *DBI::errstr;
-    *DBIx::Squirrel::rows                         = *DBI::rows;
-    *DBIx::Squirrel::lasth                        = *DBI::lasth;
-    *DBIx::Squirrel::state                        = *DBI::state;
-    *DBIx::Squirrel::connect                      = *DBIx::Squirrel::dr::connect;
-    *DBIx::Squirrel::connect_cached               = *DBIx::Squirrel::dr::connect_cached;
-    *DBIx::Squirrel::FINISH_ACTIVE_BEFORE_EXECUTE = *DBIx::Squirrel::st::FINISH_ACTIVE_BEFORE_EXECUTE;
-    *DBIx::Squirrel::DEFAULT_SLICE                = *DBIx::Squirrel::it::DEFAULT_SLICE;
-    *DBIx::Squirrel::DEFAULT_MAXROWS              = *DBIx::Squirrel::it::DEFAULT_MAXROWS;
-    *DBIx::Squirrel::BUF_MULT                     = *DBIx::Squirrel::it::BUF_MULT;
-    *DBIx::Squirrel::BUF_MAXROWS                  = *DBIx::Squirrel::it::BUF_MAXROWS;
-    *DBIx::Squirrel::NORMALISE_SQL                = *DBIx::Squirrel::util::NORMALISE_SQL;
-    *DBIx::Squirrel::NORMALIZE_SQL                = *DBIx::Squirrel::util::NORMALISE_SQL;
-
-    unless (defined $DBIx::Squirrel::VERSION) {
-        my $v = "1.3.0";
-        *DBIx::Squirrel::VERSION = \$v;
-    }
-}
-
-use constant E_BAD_ENT_BIND     => 'Cannot associate with an invalid object';
-use constant E_EXP_HASH_ARR_REF => 'Expected a reference to a HASH or ARRAY';
-
-sub _partition_imports_into_helpers_and_dbi_imports {
-    my $class = shift;
-    my(@helpers, @dbi);
-    while (@_) {
-        my $symbol = shift;
-        next unless defined($symbol);
-        if ($symbol eq 'database_entities') {
-            my $symbols = shift;
-            if (UNIVERSAL::isa($symbols, 'ARRAY')) {
-                push @helpers, @{$symbols};
-            }
-        }
-        elsif ($symbol eq 'database_entity') {
-            $symbol = shift;
-            if (defined($symbol) and not ref($symbol)) {
-                push @helpers, $symbol;
-            }
-        }
-        else {
-            push @dbi, $symbol;
-        }
-    }
-    return (\@helpers, \@dbi);
-}
-
-sub import {
-    no strict 'refs';    ## no critic
-    my $class  = shift;
-    my $caller = caller;
-    my($helpers, $dbi) = $class->_partition_imports_into_helpers_and_dbi_imports(@_);
-    for my $name (@{$helpers}) {
-        my $symbol = $class . '::' . $name;
-        my $helper = sub {
-            unless (defined(${$symbol})) {
-                if (@_) {
-                    throw E_BAD_ENT_BIND
-                      unless UNIVERSAL::isa($_[0], 'DBI::db')
-                      or UNIVERSAL::isa($_[0], 'DBI::st')
-                      or UNIVERSAL::isa($_[0], 'DBIx::Squirrel::it');
-                    ${$symbol} = shift;
-                    return ${$symbol};
-                }
-            }
-            return unless defined(${$symbol});
-            if (@_) {
-                my @params = do {
-                    if (@_ == 1 && ref $_[0]) {
-                        if (reftype($_[0]) eq 'ARRAY') {
-                            @{+shift};
-                        }
-                        elsif (reftype($_[0]) eq 'HASH') {
-                            %{+shift};
-                        }
-                        else {
-                            throw E_EXP_HASH_ARR_REF;
-                        }
-                    }
-                    else {
-                        @_;
-                    }
-                };
-                if (UNIVERSAL::isa(${$symbol}, 'DBI::db')) {
-                    return ${$symbol}->prepare(@params);
-                }
-                elsif (UNIVERSAL::isa(${$symbol}, 'DBI::st')) {
-                    return ${$symbol}->execute(@params);
-                }
-                elsif (UNIVERSAL::isa(${$symbol}, 'DBIx::Squirrel::it')) {
-                    return ${$symbol}->iterate(@params);
-                }
-                else {
-                    # ok - no worries
-                }
-            }
-            return ${$symbol};
-        };
-        *{$symbol} = subname($name => $helper);
-        *{$caller . '::' . $name} = subname($caller . '::' . $name => \&{$symbol})
-          unless defined(&{$caller . '::' . $name});
-    }
-    if (@{$dbi}) {
-        DBI->import(@{$dbi});
-        @_ = ('DBIx::Squirrel', @{$dbi});
-        goto &Exporter::import;
-    }
-    return $class;
-}
-
-1;
-__END__
-
-=pod
-
-=encoding UTF-8
 
 =head1 DESCRIPTION
 
@@ -812,7 +805,7 @@ Each stage of a transformation receives the latest version of the result via
 the argument-list (C<$_[0]> to be precise). For the sake of convenience (and
 for convention), this result is also available as C<$_>. If you prefer to
 rely on something like C<$_>, but would like something much less ephemeral,
-just C<use DBIx::Squirrel::util 'result'> and use the C<result> function
+just C<use DBIx::Squirrel::Utils 'result'> and use the C<result> function
 inside your transformation stage.
 
 Handing off to the next stage, or the caller, is with an explicit C<return>
@@ -1214,12 +1207,12 @@ It is the normalised form of the statement that is cached by the C<DBI>.
     $itor = $sth->results(\%bind_mappings, @transforms)
                 or die ...;
 
-=head3 Iterator Objects
+=head3 Iterator Methods
 
 =head4 C<all>
 
-    @ary = $itor->all();
-    $ary_ref | undef = $itor->all();
+    @results = $itor->all();
+    $results_or_undef = $itor->all();
 
 Executes the iterator's underlying statement handle object.
 
@@ -1230,105 +1223,420 @@ When called in scalar-context, this method returns a reference to
 an array of all matching row objects. Where no rows are matched,
 C<undef> would be returned.
 
+=head4 C<buffer_size>
+
+Deprecated alias for C<cache_size>.
+
+=head4 C<buffer_size_slice>
+
+Deprecated alias for C<cache_size_slice>.
+
+=head4 C<cache_size>
+
+    $cache_size = $itor->cache_size();
+    $itor = $itor->cache_size($cache_size);
+
+May be used to determine how many results the iterator makes available to
+fetch following each trip to the database.
+
+To change this property, simply provide the new value as the argument. When
+used to change the buffer-size, a reference to the iterator is returned.
+
+Normally, an iterator's buffer-size is set to a low initial value, greater
+than 1. It is usually gradually increased by a factor of two, up to a maximum,
+after every database read. This is done to improve performance when fetching
+large result sets.
+
+Manually setting the iterator's buffer-size has the side-effect of fixing it
+at that value, preventing the kind of automatic adjustment described above.
+
+The following package globals define the relevant default settings:
+
+    $DBIx::Squirrel::Iterator::DEFAULT_CACHE_SIZE = 2;   # initial buffer-size
+    $DBIx::Squirrel::Iterator::CACHE_SIZE_LIMIT   = 64;  # maximum buffer-size
+
+=head4 C<cache_size_slice>
+
+    ($cache_size, $slice) = $itor->cache_size_slice();
+    $itor = $itor->cache_size_slice($slice);
+    $itor = $itor->cache_size_slice($cache_size);
+    $itor = $itor->cache_size_slice($slice, $cache_size);
+    $itor = $itor->cache_size_slice($cache_size, $slice);
+
+May be used to determine (a) how the iterator slices the results it fetches
+from the database, and (b) how many results it makes available to fetch
+following each trip to the database.
+
+When called with no arguments, a list comprised of the following two iterator
+properties is returned:
+
+=over
+
+=item * C<$cache_size>
+
+The current size of the results buffer. That is, the current maximum number of
+results that are processed and ready to fetch after each trip to the database.
+
+=item * C<$slice>
+
+The how the iterator slices results fetched from the database. This may be an
+ARRAYREF or a HASHREF.
+
+=back
+
+To change these properties, simply provide the new values in the argument
+list. When used to change these properties, a reference to the iterator is
+returned.
+
+Normally, an iterator's buffer-size is set to a low initial value, greater
+than 1. It is usually gradually increased by a factor of two, up to a maximum,
+after every database read. This is done to improve performance when fetching
+large result sets.
+
+Manually setting the iterator's buffer-size has the side-effect of fixing it
+at that value, preventing the kind of automatic adjustment described above.
+
+The following package globals define the relevant default settings:
+
+    $DBIx::Squirrel::Iterator::DEFAULT_SLICE       = [];  # slicing strategy
+    $DBIx::Squirrel::Iterator::DEFAULT_CACHE_SIZE = 2;   # initial buffer-size
+    $DBIx::Squirrel::Iterator::CACHE_SIZE_LIMIT   = 64;  # maximum buffer-size
+
 =head4 C<count>
 
-Alias I<(see C<countnext>)>.
+    $count = $itor->count();
 
-=head4 C<countall>
+Returns the total number of rows in the result set.
 
-    $count = $itor->countall();
+If the iterator's statement has not yet been executed, it will be, and C<undef>
+will be returned if the statement was not executed successfully.
 
-Returns the number of matching rows by calculating the size of the
-array that would be created when calling the C<all> method.
+Any results remaining to be fetched are then fetched, counted and discarded,
+and the final count is returned.
 
-Of the two counting methods, this may be the fastest but it is also
-the most memory-hungry.
+I<B<BEWARE> that you should not use C<next> after this method has been used!>
 
-=head4 C<countnext>
+=head4 C<count_fetched>
 
-    $count = $itor->countnext();
+    $count = $itor->count_fetched();
 
-Returns the number of matching rows by counting the number of times
-C<next> can be called.
+Returns the number of results fetched so far.
 
-Of the two counting methods, this may be the least memory-hungry.
-
-=head4 C<done>
-
-Alias I<(see C<finished>)>.
-
-=head4 C<execute>
-
-    $rv = $itor->execute());
-
-Executes the iterator's underlying statemeent handle object.
-
-=head4 C<executed>
-
-    $bool = $itor->executed();
-
-Returns true (C<!!1>) if the underlying statement has been executed,
-otherwise it returns false (C<!!0>).
-
-=head4 C<find>
-
-=head4 C<finish>
-
-=head4 C<finished>
-
-    $bool = $itor->finished();
-
-Returns true (C<!!1>) if all matching rows have been fetched,
-otherwise it returns false (C<!!0>).
+If the iterator's statement has not yet been executed, it will be. Zero will
+be returned if the statement executed successfully, otherwise C<undef> is
+returned.
 
 =head4 C<first>
 
+    $result = $itor->first();
+
+Returns the first result in the result set, or C<undef> if there were no
+results.
+
+If the iterator's statement has not yet been executed, it will be, and C<undef>
+will be returned if the statement was not executed successfully.
+
+If the first result hasn't yet been fetched, it will be and the first result
+is fetched and cached. The cached value is returned.
+
+The result of the statement's execution will be returned.
+
 =head4 C<iterate>
 
-    $itor | undef = $itor->iterate());
+    $itor_or_undef = $itor->iterate()
+    $itor_or_undef = $itor->iterate(@bind_values)
+    $itor_or_undef = $itor->iterate(@transforms)
+    $itor_or_undef = $itor->iterate(@bind_values, @transforms)
+    $itor_or_undef = $itor->iterate(%bind_mappings)
+    $itor_or_undef = $itor->iterate(%bind_mappings, @transforms)
+    $itor_or_undef = $itor->iterate(\@bind_values)
+    $itor_or_undef = $itor->iterate([@bind_values, @transforms])
+    $itor_or_undef = $itor->iterate(\%bind_mappings)
+    $itor_or_undef = $itor->iterate(\%bind_mappings, @transforms)
 
-Executes the iterator's underlying statemeent handle object,
-returning the iterator object reference if execution was successful,
-or C<undef> if not.
+Executes the iterator's underlying statement, as well as resetting the
+iterator's internal state.
+
+When called with no arguments, any bind-values and transformations passed to
+the iterator at the time of construction will be honoured.
+
+A reference to the iterator is returned if the statement was successfully
+executed, otherwise the method returns C<undef>.
+
+=head4 C<last>
+
+    $result = $itor->last();
+
+Returns the last result in the result set.
+
+If the iterator's statement has not yet been executed, it will be, and C<undef>
+will be returned if the statement was not executed successfully.
+
+Any results remaining to be fetched are then fetched and discarded, and the
+last result fetched is returned.
+
+I<B<BEWARE> that you should not use C<next> after this method has been used!>
+
+=head4 C<last_fetched>
+
+    $result = $itor->last_fetched();
+
+Returns the last result fetched.
+
+If the iterator's statement has not yet been executed, it will be, then
+C<undef> is returned regardless of the statement execution's outcome.
+
+If the statement was previously executed then the last result fetched is
+always cached. The cached value is returned.
 
 =head4 C<next>
 
+    $result = $itor->next();
+
+Returns the next result in the result set.
+
+If the iterator's statement has not yet been executed, it will be, and C<undef>
+will be returned if the statement was not executed successfully.
+
+There are two potential side-effects that could result from a call to C<next>:
+
+=over
+
+=item *
+
+The first time it is called, the result returned will be cached and returned in
+any subsequent call to C<first>.
+
+=item *
+
+Every time it is called, the most recent result returned will be cached and
+returned in any call to C<last_fetched>, or C<last> if it was the final result
+in the result set.
+
+=back
+
 =head4 C<one>
 
-Alias I<(see C<single>)>.
+Alias (see C<single>).
 
 =head4 C<remaining>
 
-    @ary = $itor->remaining();
-    $ary_ref | undef = $itor->remaining();
-
-When called in list-context, the C<all> method returns an array
-of all matching row objects remaining to be fetched.
-
-When called in scalar-context, this method returns a reference to
-an array of all matching row objects remaining to be fetched. Where
-no rows are matched, C<undef> would be returned.
+    @results = $itor->remaining();
+    $results_or_undef = $itor->remaining;
 
 =head4 C<reset>
 
+    $itor = $itor->reset();
+    $itor = $itor->reset($slice);
+    $itor = $itor->reset($cache_size);
+    $itor = $itor->reset($slice, $cache_size);
+    $itor = $itor->reset($cache_size, $slice);
+
+Executes the iterator's underlying statement handle object with the current
+bind-values, resetting any internal state.
+
+This method may also be used to set the statement cache size and/or slicing
+strategy in the same call.
+
+A reference to the iterator is returned, regardless of execution outcome.
+
 =head4 C<rows>
 
-    $rv = $itor->rows();
+    $rows = $itor->rows();
 
 Returns the number of rows aftected by non-SELECT statements.
 
-=head4 C<single>
+=head4 C<single> (or C<one>)
 
-=head4 C<statement_handle>
+    $result = $itor->single();
 
-    $sth = $itor->statement_handle();
+Returns the first result in the result set, or C<undef> if there were no
+results.
 
-Returns a reference to the statement handle object associated with
-the iterator.
+If the iterator's statement has not yet been executed, it will be, and C<undef>
+will be returned if the statement was not executed successfully.
+
+If the first result hasn't yet been fetched, it will be and the first result
+is fetched and cached. The cached value is returned.
+
+If the result returned is one of many buffered, a warning will be issued:
+
+    Query would yield more than one result...
+
+The warning is a reminder to include a LIMIT 1 constraint in the statement.
+
+=head4 C<slice>
+
+    $slice = $itor->slice();
+    $itor = $itor->slice($slice);
+
+May be used to determine how the iterator slices the results it fetches
+from the database. This may be an ARRAYREF or a HASHREF.
+
+To change this property, simply provide the new value as the argument. When
+used to change the slicing strategy, a reference to the iterator is returned.
+
+The following package global defines the default setting:
+
+    $DBIx::Squirrel::Iterator::DEFAULT_SLICE       = [];  # slicing strategy
+
+=head4 C<slice_buffer_size>
+
+Deprecated alias for C<slice_cache_size>.
+
+=head4 C<slice_cache_size>
+
+    ($slice, $cache_size) = $itor->slice_cache_size();
+    $itor = $itor->slice_cache_size($slice);
+    $itor = $itor->slice_cache_size($cache_size);
+    $itor = $itor->slice_cache_size($slice, $cache_size);
+    $itor = $itor->slice_cache_size($cache_size, $slice);
+
+May be used to determine (a) how the iterator slices the results it fetches
+from the database, and (b) how many results it makes available to fetch
+following each trip to the database.
+
+When called with no arguments, a list comprised of the following two iterator
+properties is returned:
+
+=over
+
+=item * C<$slice>
+
+The how the iterator slices results fetched from the database. This may be an
+ARRAYREF or a HASHREF.
+
+=item * C<$cache_size>
+
+The current size of the results buffer. That is, the current maximum number of
+results that are processed and ready to fetch after each trip to the database.
+
+=back
+
+To change these properties, simply provide the new values in the argument
+list. When used to change these properties, a reference to the iterator is
+returned.
+
+Normally, an iterator's buffer-size is set to a low initial value, greater
+than 1. It is usually gradually increased by a factor of two, up to a maximum,
+after every database read. This is done to improve performance when fetching
+large result sets.
+
+Manually setting the iterator's buffer-size has the side-effect of fixing it
+at that value, preventing the kind of automatic adjustment described above.
+
+The following package globals define the relevant default settings:
+
+    $DBIx::Squirrel::Iterator::DEFAULT_SLICE       = [];  # slicing strategy
+    $DBIx::Squirrel::Iterator::DEFAULT_CACHE_SIZE = 2;   # initial buffer-size
+    $DBIx::Squirrel::Iterator::CACHE_SIZE_LIMIT   = 64;  # maximum buffer-size
+
+=head4 C<start>
+
+    $rv_or_undef = $itor->start()
+    $rv_or_undef = $itor->start(@bind_values)
+    $rv_or_undef = $itor->start(@transforms)
+    $rv_or_undef = $itor->start(@bind_values, @transforms)
+    $rv_or_undef = $itor->start(%bind_mappings)
+    $rv_or_undef = $itor->start(%bind_mappings, @transforms)
+    $rv_or_undef = $itor->start(\@bind_values)
+    $rv_or_undef = $itor->start([@bind_values, @transforms])
+    $rv_or_undef = $itor->start(\%bind_mappings)
+    $rv_or_undef = $itor->start(\%bind_mappings, @transforms)
+
+Culminates in the iterator's underlying statement being executed, after
+resetting the iterator's internal state.
+
+When called with no arguments, any bind-values and transformations passed to
+the iterator at the time of construction are used.
 
 =head4 C<sth>
 
-Alias I<(see C<statement_handle>)>.
+    $sth = $itor->sth();
+
+Returns the iterator's underlying statement handle object.
+
+=head3 Iterator Exports
+
+The C<DBIx::Squirrel::Iterator> package exports a number of subroutines that
+may be used within the stages of a transformation pipeline. These provide
+information about the current transformation context.
+
+=head4 C<database>
+
+    my $dbh = database();
+
+Returns a reference to the current iterator's database handle object.
+
+=head4 C<iterator>
+
+    my $itor = iterator();
+
+Returns a reference to the current iterator.
+
+=head4 C<result>
+
+    my $result = result();
+
+Returns the result as it was when it entered the I<current stage> of the
+transformation pipeline.
+
+There are alternative ways to get at this value without having to pollute
+your namespace, although C<result> is less cryptic:
+
+=over
+
+=item *
+
+It is the first element of the C<@_> list. Use C<$_[0]>, or C<shift> the
+value from the start of the list.
+
+=item *
+
+It is also presented in the ephemeral C<$_> variable. It should be consumed
+quickly, before it changes.
+
+=back
+
+=head4 C<result_current>
+
+An alias (see C<result>).
+
+=head4 C<result_first>
+
+    my $result = result_first();
+
+Returns the first result fetched.
+
+=head4 C<result_prev>
+
+    my $result = result_previous();
+
+Returns the previous result, if there was one.
+
+=head4 C<result_previous>
+
+An alias (see C<result_prev>).
+
+=head4 C<result_offset>
+
+    my $offset = result_offset();
+
+Returns the result's zero-based offset, effectively the number of results
+fetched, less one.
+
+=head4 C<result_original>
+
+    my $result = result_original();
+
+Returns the result as it was when it entered the I<first stage> of the current
+transformation pipeline.
+
+=head4 C<statement>
+
+    my $sth = statement();
+
+Returns a reference to the current iterator's statement handle object.
 
 =head1 COPYRIGHT AND LICENSE
 
